@@ -1,4 +1,4 @@
-use std::sync::Arc;
+use std::{str::FromStr, sync::Arc};
 
 use http::StatusCode;
 use once_cell::sync::Lazy;
@@ -6,7 +6,8 @@ use regex::Regex;
 
 use super::tree::Tree;
 use crate::{
-    http::Method, Body, Endpoint, EndpointExt, IntoEndpoint, IntoResponse, Request, Response,
+    http::{uri::PathAndQuery, Method, Uri},
+    Body, Endpoint, EndpointExt, IntoEndpoint, IntoResponse, Request, Response,
 };
 
 /// Routing object
@@ -78,6 +79,7 @@ impl Route {
 
         struct Nest<T> {
             inner: T,
+            root: bool,
             prefix_len: usize,
         }
 
@@ -86,39 +88,24 @@ impl Route {
             type Output = Response;
 
             async fn call(&self, mut req: Request) -> Self::Output {
-                let idx = req.state().match_params.len() - 1;
-                let (name, _) = req.state_mut().match_params.remove(idx);
-                assert_eq!(name, "--poem-rest");
-                req.set_uri(
-                    http::uri::Builder::new()
-                        .path_and_query(
-                            &req.uri().path_and_query().unwrap().as_str()[self.prefix_len..],
+                if !self.root {
+                    let idx = req.state().match_params.len() - 1;
+                    let (name, _) = req.state_mut().match_params.remove(idx);
+                    assert_eq!(name, "--poem-rest");
+                }
+
+                let new_uri = {
+                    let uri = std::mem::take(req.uri_mut());
+                    let mut uri_parts = uri.into_parts();
+                    uri_parts.path_and_query = Some(
+                        PathAndQuery::from_str(
+                            &uri_parts.path_and_query.as_ref().unwrap().as_str()[self.prefix_len..],
                         )
-                        .build()
                         .unwrap(),
-                );
-                self.inner.call(req).await.into_response()
-            }
-        }
-
-        struct Root<T> {
-            inner: T,
-            prefix_len: usize,
-        }
-
-        #[async_trait::async_trait]
-        impl<E: Endpoint> Endpoint for Root<E> {
-            type Output = Response;
-
-            async fn call(&self, mut req: Request) -> Self::Output {
-                req.set_uri(
-                    http::uri::Builder::new()
-                        .path_and_query(
-                            &req.uri().path_and_query().unwrap().as_str()[self.prefix_len..],
-                        )
-                        .build()
-                        .unwrap(),
-                );
+                    );
+                    Uri::from_parts(uri_parts).unwrap()
+                };
+                *req.uri_mut() = new_uri;
                 self.inner.call(req).await.into_response()
             }
         }
@@ -136,13 +123,15 @@ impl Route {
             &format!("{}*--poem-rest", path),
             Box::new(Nest {
                 inner: ep.clone(),
+                root: false,
                 prefix_len,
             }),
         );
         self.router.add(
             &path[..path.len() - 1],
-            Box::new(Root {
+            Box::new(Nest {
                 inner: ep,
+                root: true,
                 prefix_len,
             }),
         );
